/*
Copyright (c) 2003-2004, Gregory Kuhlmann, Peter Stone, Justin Lallinger,
University of Texas at Austin
All right reserved

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the University of Texas nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "WorldModel.h"
#include "SoccerTypes.h"
#include "ClangRBInterface.h"

WorldModel *wm;
WorldModel *RBWM;

bool   RBGetPlayerKnown(CLTeam t, int p)
{
	ObjectT plr = OBJECT_TEAMMATE_1;
	if (t == our)
		plr = SoccerTypes::getTeammateObjectFromIndex(p-1);
	else
		plr = SoccerTypes::getOpponentObjectFromIndex(p-1);

	return (plr==wm->getAgentObjectType()) ||
	  (wm->getConfidence(plr) > 0.88);
}


double RBGetPlayerX( CLTeam t, int p )
{
  ObjectT plr = OBJECT_TEAMMATE_1;
  if ( t == our )
    plr = SoccerTypes::getTeammateObjectFromIndex( p - 1 );
  else
    plr = SoccerTypes::getOpponentObjectFromIndex( p - 1 );

  VecPosition pos = wm->getGlobalPosition( plr );
  return pos.getX();
}

double RBGetPlayerY( CLTeam t, int p )
{
  ObjectT plr = OBJECT_TEAMMATE_1;
  if ( t == our )
    plr = SoccerTypes::getTeammateObjectFromIndex( p - 1 );
  else
    plr = SoccerTypes::getOpponentObjectFromIndex( p - 1 );

  VecPosition pos = wm->getGlobalPosition( plr );
  return pos.getY();
}

int RBGetBallOwnerUnum()
{
  return SoccerTypes::getIndex( wm->getBallOwner() ) + 1;
}

CLTeam RBGetBallOwnerTeam()
{
  if ( SoccerTypes::isTeammate( wm->getBallOwner() ) )
    return our;
  else
    return opp;

}

double RBGetBallX()
{
  return wm->getBallPos().getX();
}

double RBGetBallY()
{
  return wm->getBallPos().getY();

}

int RBGetScore( CLTeam t )
{
  if ( t == our )
    return wm->getOurScore();
  else
    return wm->getTheirScore();

}

int RBGetTime()
{

  return wm->getCurrentCycle();
}

CLPlayMode RBGetPlayMode()
{
  SideT ourside = wm->getSide();
  switch ( wm->getPlayMode() ) {
  case PM_BEFORE_KICK_OFF:
    return bko;
    break;
  case PM_KICK_OFF_LEFT:
    if ( ourside == SIDE_LEFT )
      return ko_our;
    else
      return ko_opp;
    break;
  case PM_KICK_OFF_RIGHT:
    if ( ourside == SIDE_LEFT )
      return ko_opp;
    else
      return ko_our;
    break;
  case PM_KICK_IN_LEFT:
    if ( ourside == SIDE_LEFT )
      return ki_our;
    else
      return ki_opp;
    break;
  case PM_KICK_IN_RIGHT:
    if ( ourside == SIDE_LEFT )
      return ki_opp;
    else
      return ki_our;
    break;
  case PM_CORNER_KICK_LEFT:
    if ( ourside == SIDE_LEFT )
      return ck_our;
    else
      return ck_opp;
    break;
  case PM_CORNER_KICK_RIGHT:
    if ( ourside == SIDE_LEFT )
      return ck_opp;
    else
      return ck_our;
    break;
  case PM_GOAL_KICK_LEFT:
    if ( ourside == SIDE_LEFT )
      return gk_our;
    else
      return gk_opp;
    break;
  case PM_GOAL_KICK_RIGHT:
    if ( ourside == SIDE_LEFT )
      return gk_opp;
    else
      return gk_our;
    break;
  case PM_GOAL_LEFT:
    if ( ourside == SIDE_LEFT )
      return ag_our;
    else
      return ag_opp;
    break;
  case PM_GOAL_RIGHT:
    if ( ourside == SIDE_LEFT )
      return ag_opp;
    else
      return ag_our;
    break;
  case PM_FREE_KICK_FAULT_LEFT:
    if ( ourside == SIDE_LEFT )
      return fk_our;
    else
      return fk_opp;
    break;
  case PM_FREE_KICK_FAULT_RIGHT:
    if ( ourside == SIDE_LEFT )
      return fk_opp;
    else
      return fk_our;
    break;
  case PM_FREE_KICK_LEFT:
  case PM_INDIRECT_FREE_KICK_LEFT:
    if ( ourside == SIDE_LEFT )
      return fk_our;
    else
      return fk_opp;
    break;
  case PM_FREE_KICK_RIGHT:
  case PM_INDIRECT_FREE_KICK_RIGHT:
    if ( ourside == SIDE_LEFT )
      return fk_opp;
    else
      return fk_our;
    break;
  case PM_BACK_PASS_LEFT:
  case PM_BACK_PASS_RIGHT:
  case PM_OFFSIDE_LEFT:
    if ( ourside == SIDE_LEFT )
      return fk_opp;
    else
      return fk_our;
    break;
  case PM_OFFSIDE_RIGHT:
    if ( ourside == SIDE_RIGHT )
      return fk_opp;
    else
      return fk_our;
    break;
  case PM_PLAY_ON:
    return play_on;
    break;
  case PM_TIME_OVER:
    //case PM_FROZEN:
    //case PM_QUIT:
  
    //case PM_ILLEGAL:
  default:
    return time_over;
    break;
  }
  return play_on;
}

bool RBReverseCoords()
{
//      return wm->getSide() == SIDE_RIGHT;
  return false;
}

void RuleBaseInit( WorldModel * w )
{
  wm = w;
  RBWM = w;
  rulebase = new RuleBase;

  CLGetPlayerKnown = RBGetPlayerKnown;
  CLGetPlayerY = RBGetPlayerY;
  CLGetPlayerX = RBGetPlayerX;
  CLGetBallOwnerUnum = RBGetBallOwnerUnum;
  CLGetBallOwnerTeam = RBGetBallOwnerTeam;
  CLGetBallX = RBGetBallX;
  CLGetBallY = RBGetBallY;
  CLGetScore = RBGetScore;
  CLGetTime = RBGetTime;
  CLGetPlayMode = RBGetPlayMode;
  CLReverseCoords = RBReverseCoords;
}

void RuleBaseCleanup()
{
  delete rulebase;
}
