/*
Copyright (c) 2003-2004, Gregory Kuhlmann, Peter Stone, Justin Lallinger,
University of Texas at Austin
All right reserved

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the University of Texas nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef _CLANG_RULE_
#define _CLANG_RULE_

#include <iostream>
#include "ClangTypes.h"
#include "ClangCondition.h"
#include "ClangDirective.h"

struct USlist
{
  UnumSet *s;
  USlist *next;
} ;

class Rule;
struct idList
{
  idList()
  {
    r = NULL;
    next = NULL;
  }
  
  ~idList()
  {
    delete r;
  }
  
  CLangVar *r;
  idList *next;
} ;

struct ruleList
{
  ruleList();
  ~ruleList();
  Rule *r;
  ruleList *next;
} ;

class Rule
{
 public:
  Rule()
  {
    inst = NULL;
  }
  
  virtual ~ Rule() {}
  virtual void Reset() {}
  virtual void AddDirectivesToRulebase( bool direcm ) {}
  virtual void Output( ostream & os, bool dispvar = true ) {}

  void InstantiateVariable( UnumSet * v );
  UnumSet *GetInstantiatedVariable( CLangVar * v );
  void ClearInstantiations();

 private:
  USlist * inst;
} ;

// Basic "Condition -> DirectiveList rule"

class CondDirRule: public Rule
{
 public:
  CondDirRule( Condition * c, directiveList * d );
  virtual ~ CondDirRule();
  virtual void Reset();
  virtual void AddDirectivesToRulebase( bool direcm );
  virtual void Output( ostream & os, bool dispvar = true );

 private:
  Condition * c;
  directiveList *d;
  bool used;
} ;

// "Condition -> Rulelist" Rule
class CondRuleRule: public Rule
{
 public:
  CondRuleRule( Condition * c, ruleList * l );
  virtual ~CondRuleRule();
  virtual void Reset();
  virtual void AddDirectivesToRulebase( bool direcm );
  virtual void Output( ostream & os, bool dispvar = true );

 private:
  Condition * c;
  ruleList *l;
  bool used;
} ;

// "ID_LIST" Rule
class IDListRule: public Rule
{
 public:
  IDListRule( idList * l );
  virtual ~ IDListRule();
  virtual void Reset();
  virtual void AddDirectivesToRulebase( bool direcm );
  virtual void Output( ostream & os, bool dispvar = true );

private:
  idList * l;
  bool used;
} ;

#endif
