/*
Copyright (c) 2003-2004, Gregory Kuhlmann, Peter Stone, Justin Lallinger,
University of Texas at Austin
All right reserved

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the University of Texas nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef _CLANG_RULEBASE_
#define _CLANG_RULEBASE_

#include "ClangTypes.h"
#include "ClangRule.h"
#include "ClangDirective.h"
#include "ClangMsg.h"

enum CLVariableType { 
  UNDEFSTR, 
  UNDEFVAR, 
  DEFREGION, 
  DEFPTREG, 
  DEFCOND, 
  DEFACTION, 
  DEFRULE,
  DEFDIR 
} ;

#define UPDATE_ALL 	3
#define UPDATE_DEFINES	2
#define UPDATE_RULES	1

struct RBlist
{
  Directive *dir;
  RBlist *next;
} ;

struct UpdateMsg
{
  UpdateMsg()
  {
    var = NULL;
  }
  
  ~UpdateMsg()
  {
    delete var;
  }
  
  CLangVar *var;
  bool onoff;
  bool isrule;
  bool direcmode;
  definemsgtype type;
  UpdateMsg *next;
} ;

class RuleBase
{
 public:
  RuleBase() {
    activedirecs = NULL;
    activemodels = NULL;
    regs = NULL;
    acts = NULL;
    dirs = NULL;
    conds = NULL;
    rules = NULL;
    coachmode = false;
    defupdate = NULL;
    defupdatel = NULL;
    ruleupdate = NULL;
    ruleupdatel = NULL;
  }

  ~RuleBase();
  
  void DeleteLists();
  
  RBlist *GetActiveDirecs()
  {
    return activedirecs;
  }
  
  RBlist *GetActiveModels()
  {
    return activemodels;
  }
  
  void UpdateLists();
  bool HearMsg( char *str );
  void SetCoachMode( bool mode = true );

  void AddToActiveDirecs( Directive * d );
  void AddToActiveModels( Directive * d );

  Region *ResolveRegion( CLangVar * v );
  PointRegion *ResolvePtRegion( CLangVar * v );
  Directive *ResolveDirective( CLangVar * v );
  Action *ResolveAction( CLangVar * v );
  Rule *ResolveRule( CLangVar * v );
  Condition *ResolveCondition( CLangVar * v );
  bool IsDefinedVar( CLangVar * v );
  bool IsDefinedStr( CLangVar * v );
  bool IsUndefinedVar( CLangVar * v );
  bool IsUndefinedStr( CLangVar * v );
  bool IsRegion( CLangVar * v );
  bool IsCondition( CLangVar * v );
  bool IsDirective( CLangVar * v );
  bool IsAction( CLangVar * v );
  bool IsRule( CLangVar * v );
  bool IsPointRegion( CLangVar * v );

  bool IsRuleOn( CLangVar * v );
  bool IsRuleOff( CLangVar * v );
  bool IsRuleModel( CLangVar * v );
  bool IsRuleDirec( CLangVar * v );

  void ChangeRuleState( char *v, bool onoff );
  void ChangeRuleState( CLangVar * v, bool onoff );
  void ChangeRuleState( bool onoff );
  void ChangeRuleState( idList * r, bool onoff );

  void DeleteRule();
  void DeleteRule( CLangVar * v );
  void DeleteRule( idList * rl );

  bool NeedsUpdate( int mode = UPDATE_ALL );

  void HearDefineMsg( definemsg * m );
  CLangVar *GetRuleName( Rule * r );
  void LoadFromFile( char *str, int *, int * );
  void LoadFromFile( istream & f, int *, int * );
  
 private:

  definemsg * acts;
  definemsg *dirs;
  definemsg *regs;
  definemsg *conds;
  definemsg *rules;

  RBlist *activemodels;
  RBlist *activedirecs;

  UpdateMsg *defupdate;
  UpdateMsg *defupdatel;
  UpdateMsg *ruleupdate;
  UpdateMsg *ruleupdatel;
  bool coachmode;
} ;

#endif
